#!/usr/bin/python3
# -*- coding: utf-8 -*-
'''
________________________________________________________________________
 a_db_anlegen.py                
 This file is free software under    
     the terms of the GPL without any warranty - see the file COPYING 
 Diese Datei ist Freie Software gemäß
         der GPL ohne jegliche Garantie ist - siehe die Datei COPYING 
________________________________________________________________________

 a_db_anlegen.py legt die notwendige Datenbank mit allen Tabellen an.
 
 Es kann mehrfach aufgerufen werden! Bei Updates ist es aufzurufen, damit 
 Tabellenänderungen eingepflegt werden.
 
 Um den admin mit dem Standardpasswort "pass8" neu anzulegen, ist 
 dieses Skript mit dem Parameter "resetadmin" aufzurufen.
 
________________________________________________________________________
'''

import sys
import codecs
import time, datetime
import subprocess

import a_db
import hashlib

def db_create():
    global db_name
    try:
        a_db.db_verbinden(db_name)
    except:
        global doprint
        doprint=True
        prt('''Content-Type: text/html\n\n<!DOCTYPE html><html> \n
        <head><link rel="stylesheet" href="/a.css" type="text/css" />
            <meta http-equiv="content-type" content="text/html; charset=utf-8" />\n
            <meta http-equiv="cache-control" content="no-cache">
        </head><body>
            <h2>Problem: Konnte meine Datenbank nicht automatisch anlegen.<br>
            Lösung: Starten Sie a_db_anlegen.py im cgi-bin-Verzeichnis!</h2><br>
            Ursache: Es existiert noch keine Datenbank  '''+db_name+'''<br>
            Unter dem Webserver Apache wird diese automatisch angelegt; <br>
            arbeitet man aber mit localserver.py, so fehlt dem Webserver (aus Sicherheitsgründen) das Recht Dateien anzulegen.<br>Dies muss deshalb einmalig per Hand vorgenommen werden.
        </body></html>''')
        exit(-1)
    db_create_tables()

def db_create_tables():
    
    a_db.execute('''CREATE TABLE IF NOT EXISTS absenzen (
                            pid INTEGER,
                            status INTEGER,
                            attestpflicht INTEGER,
                            datum TEXT, 
                            kommentar TEXT,
                            PRIMARY KEY (pid, datum) ) ''')
    a_db.execute('''CREATE TABLE IF NOT EXISTS absenzenundo (
                            sid INTEGER,
                            pid INTEGER,
                            status INTEGER,
                            attestpflicht INTEGER,
                            datum TEXT, 
                            kommentar TEXT,
                            neuerkommentar TEXT,
                            PRIMARY KEY (sid, pid, datum) ) ''')
    
    a_db.execute('''CREATE TABLE IF NOT EXISTS personen (
                            pid INTEGER PRIMARY KEY AUTOINCREMENT,
                            js INTEGER,
                            klasse TEXT,
                            apdatum TEXT,
                            name TEXT,
                            sortiername TEXT,
                            zugriffscode TEXT) ''')
    
    a_db.execute('''CREATE TABLE IF NOT EXISTS kurse (
                            kbez TEXT PRIMARY KEY,
                            kname TEXT,
                            klehrer TEXT) ''')
    
    a_db.execute('''CREATE TABLE IF NOT EXISTS klassen (
                            name TEXT PRIMARY KEY,
                            js INTEGER,
                            klassleiter TEXT) ''')
    
    a_db.execute('''CREATE TABLE IF NOT EXISTS p2k (
                            pid INTEGER,
                            kbez TEXT,
                            PRIMARY KEY (pid,kbez) )''')
    
    a_db.execute('''CREATE TABLE IF NOT EXISTS sessions (
                            sid INTEGER PRIMARY KEY,
                            ablauf INTEGER,
                            aktion INTEGER,
                            rechte TEXT,
                            monat INTEGER,
                            js INTEGER,
                            highlight INTEGER,
                            kurs TEXT,
                            name TEXT,
                            klarname TEXT,
                            verlauf TEXT)   ''')
    
    a_db.execute('''CREATE TABLE IF NOT EXISTS users (
                            kuerzel TEXT UNIQUE,
                            sortiername TEXT,
                            hash TEXT,
                            rechte TEXT,
                            klarname TEXT PRIMARY KEY ) ''')
    
    a_db.execute('''CREATE TABLE IF NOT EXISTS poplog (
                            timestamp TEXT,
                            erfolg INTEGER,
                            adresse TEXT,
                            aktion TEXT) ''')
    
    a_db.execute('''CREATE TABLE IF NOT EXISTS lehrerinklasse (
                            uklarname TEXT,
                            kname TEXT) ''')
        
    a_db.execute('''CREATE TABLE IF NOT EXISTS einstellungspaerchen (
                            parameter TEXT PRIMARY KEY,
                            wert TEXT) ''')
    if not a_db.get_existenz('SELECT kuerzel FROM users WHERE kuerzel="admin"'):
        lege_admin_an()
    a_db.commit()
    
def lege_admin_an():
    kuerzel="admin"
    pwd="pass8" # initiales Passwort!
    pwws=kuerzel+"aB5EnT"+pwd  # mit Salt gegen Rainbowtables
    hash=hashlib.md5(pwws.encode('utf-8')).hexdigest() 
    rechte='9999'     # god mode
    a_db.execute('''DELETE FROM users WHERE kuerzel="admin"''')
    a_db.execute('''INSERT INTO users (kuerzel,hash,rechte,klarname) VALUES (?,?,?,?) ''' , (kuerzel, hash, rechte, "Administrator"))
    a_db.commit()
    

def prt(data):
    if doprint:
        data8=data.encode('utf-8')
        try:
            sys.stdout.buffer.write(data8)
        except:
            sys.stdout.write(data8)
    else:
        global printresult
        printresult+=data
        

    
def remove_config(parameter):
    a_db.execute('DELETE FROM einstellungspaerchen WHERE parameter=?',(parameter,))
    
def upgradedb(dbc):
    count=a_db.get_first_element("SELECT count(*) FROM sqlite_master WHERE type='table' AND name='absenzenundo' ")
    if count==0:
        db_create_tables()
    db_version=a_db.get_config('intern_db_version')
    if db_version<'2013-10-14':
        a_db.executescript('''
            ALTER TABLE absenzenundo 
                ADD COLUMN undonr INTEGER DEFAULT 0;
            ALTER TABLE absenzenundo 
                ADD COLUMN teilflag INTEGER DEFAULT 0;
            ALTER TABLE absenzen 
                ADD COLUMN teilflag INTEGER DEFAULT 0;
            UPDATE personen 
                SET klasse="Q11" WHERE js=11;
            UPDATE personen 
                SET klasse="Q12" WHERE js=12;''')
        a_db.set_config('intern_db_version','2013-10-14')
        db_version=a_db.get_config('intern_db_version')

    if db_version<'2013-11-07':
        a_db.executescript('''
            ALTER TABLE kurse 
                ADD COLUMN vlehrer TEXT DEFAULT "";''')
        a_db.set_config('intern_db_version','2013-11-07')
        db_version=a_db.get_config('intern_db_version')

    if db_version < '2013-12-29':
        a_db.executescript('''
            CREATE TABLE IF NOT EXISTS noschool (
                                name TEXT primary key,
                                erstertag TEXT,
                                letztertag TEXT);
            ALTER TABLE kurse
                ADD COLUMN wochentage INT DEFAULT 31;
            ALTER TABLE sessions
                ADD COLUMN tagesfilter INT DEFAULT 0;''')
        ns_liste=[  ("T.d.D.Einheit"    ,"9999-10-03","9999-10-03"),
                    ("T.d.Arbeit"       ,"9999-05-01","9999-05-01"),
                    ("Hlg.d.Könige"     ,"9999-01-06","9999-01-06"),
                    ("Sommer 2013"      ,"2013-07-31","2013-09-11"),
                    ("Herbst 2013"      ,"2013-10-28","2013-10-31"),
                    ("Weihnachten 2013" ,"2013-12-23","2014-01-04"),
                    ("Frühjahr 2014"    ,"2014-03-03","2014-03-07"),
                    ("Ostern 2014"      ,"2014-04-14","2014-04-26"),
                    ("Pfingsten 2014"   ,"2014-06-10","2014-06-21"),
                    ("Sommer 2014"      ,"2014-07-30","2014-09-15")]

        for ns_tupel in ns_liste:
            a_db.execute('''INSERT INTO noschool (name,erstertag,letztertag) VALUES 
                    (?,?,?)''',ns_tupel)
        a_db.set_config('intern_db_version','2013-12-29')
        db_version=a_db.get_config('intern_db_version')

    if db_version<'2014-04-21':
        a_db.executescript('''
            ALTER TABLE sessions 
                ADD COLUMN befreiungstext TEXT DEFAULT "";''')
        a_db.set_config('intern_db_version','2014-04-21')
        db_version=a_db.get_config('intern_db_version')

    if db_version<'2014-04-26':
        a_db.executescript('''
            CREATE TABLE IF NOT EXISTS gespraeche (
                                pid INTEGER,
                                datum TEXT,
                                klarname TEXT,
                                kommentar TEXT DEFAULT "" );''')
        a_db.set_config('intern_db_version','2014-04-26')
        db_version=a_db.get_config('intern_db_version')

    if db_version<'2014-05-01':
        a_db.execute('''CREATE TABLE IF NOT EXISTS gruppenrechte (
                                gruppe TEXT,
                                level INTEGER,
                                flag INTEGER DEFAULT 0,
                                PRIMARY KEY (gruppe, level) ) ''')
        a_db.set_config('intern_db_version','2014-05-01')
        import a_tool
        a_tool.manage_gruppenrechte()      
        db_version=a_db.get_config('intern_db_version')

    if db_version<'2014-05-13':
        a_db.executescript('''
            ALTER TABLE sessions 
                ADD COLUMN halbtagsflag INTEGER DEFAULT 0;''')
        a_db.set_config('intern_db_version','2014-05-13')
        db_version=a_db.get_config('intern_db_version')

    if db_version<'2014-06-09':
        try:
            a_db.executescript('''
            ALTER TABLE users 
                ADD COLUMN trechte INTEGER DEFAULT 0;''')
        except: pass
        a_db.set_config('intern_db_version','2014-06-09')
        db_version=a_db.get_config('intern_db_version')
    
    if db_version<'2014-06-20':
        a_db.executescript('''CREATE TABLE IF NOT EXISTS termine (
                                tid INTEGER PRIMARY KEY AUTOINCREMENT,
                                name TEXT,
                                kommentar TEXT DEFAULT "",
                                datum TEXT,
                                datumende TEXT DEFAULT "",
                                zeit TEXT DEFAULT "",
                                typ INTEGER DEFAULT 0,
                                status INTEGER DEFAULT 0,
                                autor TEXT,
                                betroffene TEXT DEFAULT "",
                                verantwortliche TEXT DEFAULT "",
                                history TEXT DEFAULT "" );''')
        try:
            a_db.executescript('''
            ALTER TABLE sessions 
                ADD COLUMN tbereich INTEGER DEFAULT 0;''')
        except: pass
        a_db.set_config('intern_db_version','2014-06-20')
        db_version=a_db.get_config('intern_db_version')

    if db_version<'2014-06-21':
        ns_liste=[  ("Herbst 2014"      ,"2014-10-27","2014-10-31"),
                    ("Weihnachten 2014" ,"2014-12-24","2015-01-05"),
                    ("Frühjahr 2015"    ,"2015-02-16","2015-02-20"),
                    ("Ostern 2015"      ,"2015-03-30","2015-04-11"),
                    ("Pfingsten 2015"   ,"2015-05-26","2015-06-05"),
                    ("Sommer 2015"      ,"2015-08-01","2015-09-14"),
                    ("Herbst 2015"      ,"2015-11-02","2015-11-07"),
                    ("Weihnachten 2015" ,"2015-12-24","2016-01-05"),
                    ("Frühjahr 2016"    ,"2016-02-08","2016-02-12"),
                    ("Ostern 2016"      ,"2016-03-21","2016-04-01"),
                    ("Pfingsten 2016"   ,"2016-05-17","2016-05-28"),
                    ("Sommer 2016"      ,"2016-07-30","2016-09-12")]

        for ns_tupel in ns_liste:
            a_db.execute('''INSERT INTO noschool (name,erstertag,letztertag) VALUES 
                    (?,?,?)''',ns_tupel)
        a_db.set_config('intern_db_version','2014-06-21')
        db_version=a_db.get_config('intern_db_version')
        
    if db_version<'2014-07-22':
        ns_liste=[  ("Chr. Himmelfahrt 2015"      ,"2015-05-14","2015-05-14"),
                    ("Chr. Himmelfahrt 2016"      ,"2016-05-05","2016-05-05"),
                    ("Chr. Himmelfahrt 2017"      ,"2017-05-25","2017-05-25"),
                    ("Chr. Himmelfahrt 2018"      ,"2018-05-10","2018-05-10")]
        for ns_tupel in ns_liste:
            try:a_db.execute('''INSERT INTO noschool (name,erstertag,letztertag) VALUES 
                    (?,?,?)''',ns_tupel)
            except: pass
        a_db.executescript('''
                    ALTER TABLE kurse ADD COLUMN klehrer_nr INT DEFAULT 0;
                    ALTER TABLE kurse ADD COLUMN vlehrer_nr INT DEFAULT 0;
                    ALTER TABLE sessions ADD COLUMN user_nr INT DEFAULT 0;
                    ALTER TABLE klassen ADD COLUMN klassleiter_nr INT DEFAULT 0;
                    ALTER TABLE lehrerinklasse ADD COLUMN user_nr INT DEFAULT 0;
                    ''')
        a_db.executescript('''         
                    CREATE TABLE IF NOT EXISTS users_neu (
                            user_nr INTEGER PRIMARY KEY,
                            kuerzel TEXT UNIQUE,
                            sortiername TEXT,
                            hash TEXT,
                            rechte TEXT,
                            klarname TEXT UNIQUE,
                            trechte INTEGER DEFAULT 0);
                    INSERT INTO users_neu 
                            (kuerzel,sortiername,
                            hash,rechte,
                            klarname,trechte ) 
                            SELECT  kuerzel,sortiername,
                                    hash,rechte,
                                    klarname,trechte 
                            FROM users;
                    DROP TABLE users;
                    ALTER TABLE users_neu 
                            RENAME TO users;
                            ''')
        ersetzungstabelle=a_db.get_rowdict_list('SELECT klarname,user_nr FROM users')
        for klarname,usernr in ersetzungstabelle:
            a_db.execute('UPDATE kurse SET klehrer_nr=? WHERE klehrer=?',(usernr,klarname))
            a_db.execute('UPDATE kurse SET vlehrer_nr=? WHERE vlehrer=?',(usernr,klarname))
            a_db.execute('UPDATE sessions SET user_nr=? WHERE klarname=?',(usernr,klarname))
            a_db.execute('UPDATE klassen SET klassleiter_nr=? WHERE klassleiter=?',(usernr,klarname))
            a_db.execute('UPDATE lehrerinklasse SET user_nr=? WHERE uklarname=?',(usernr,klarname))
        a_db.execute('''CREATE TABLE IF NOT EXISTS useralias (
                            aliasname TEXT PRIMARY KEY,
                            user_nr INTEGER) ''')
        a_db.executescript('''
                    CREATE TABLE IF NOT EXISTS kurse_neu (
                            kbez TEXT PRIMARY KEY,
                            kname TEXT,
                            klehrer_nr INTEGER DEFAULT 0,
                            vlehrer_nr INTEGER DEFAULT 0,
                            wochentage INTEGER DEFAULT 31);                            
                    INSERT INTO kurse_neu 
                            (kbez,kname,klehrer_nr,vlehrer_nr,wochentage) 
                            SELECT  kbez,kname,klehrer_nr,vlehrer_nr,wochentage
                            FROM kurse;
                    DROP TABLE kurse;
                    ALTER TABLE kurse_neu 
                            RENAME TO kurse;
                            ''')
        a_db.executescript('''                    
                    CREATE TABLE IF NOT EXISTS klassen_neu (
                            name TEXT PRIMARY KEY,
                            js INTEGER DEFAULT 0,
                            klassleiter_nr INTEGER DEFAULT 0);
                    INSERT INTO klassen_neu
                            (name, js, klassleiter_nr)
                            SELECT name, js, klassleiter_nr
                            FROM klassen;
                    DROP TABLE klassen;
                    ALTER TABLE klassen_neu
                            RENAME TO klassen;
                            ''')
        a_db.executescript('''         
                    CREATE TABLE IF NOT EXISTS lehrerinklasse_neu (
                            kname TEXT,
                            user_nr INTEGER);
                    INSERT INTO lehrerinklasse_neu
                            (kname,user_nr)
                            SELECT kname,user_nr
                            FROM lehrerinklasse;
                    DROP TABLE lehrerinklasse;
                    ALTER TABLE lehrerinklasse_neu
                            RENAME TO lehrerinklasse;
                            ''')
        a_db.executescript('''
                    VACUUM;        ''')

        a_db.set_config('intern_db_version','2014-07-22')
        db_version=a_db.get_config('intern_db_version')
    
    if db_version<'2014-09-07':
        a_db.execute('''ALTER TABLE sessions ADD COLUMN anzeigeschuljahr INTEGER DEFAULT 2000 ''')
        remove_config('mindestdatum-gum')
        remove_config('mindestdatum-q11')
        remove_config('mindestdatum-q12')
        a_db.set_config('intern_db_version','2014-09-07')
        db_version=a_db.get_config('intern_db_version')
    
    if db_version<'2014-09-11':
        a_db.execute('''ALTER TABLE sessions ADD COLUMN anzeigehalbjahr INTEGER DEFAULT 0 ''')
        a_db.set_config('intern_db_version','2014-09-11')
        db_version=a_db.get_config('intern_db_version')

    if db_version<'2014-10-28':
        a_db.execute('''ALTER TABLE termine ADD COLUMN tag_jb INTEGER DEFAULT 0 ''')
        a_db.execute('''ALTER TABLE termine ADD COLUMN tag_hp INTEGER DEFAULT 0 ''')
        a_db.set_config('intern_db_version','2014-10-28')
        db_version=a_db.get_config('intern_db_version')
    
    if db_version<'2014-11-09':
        a_db.execute('''ALTER TABLE termine ADD COLUMN modifiziert TEXT DEFAULT "2000-01-01" ''')
        a_db.set_config('intern_db_version','2014-11-09')
        db_version=a_db.get_config('intern_db_version')
    
    #==============================================================================================================#
    # merke: die letzte Version muss auch in a_tool.py konfiguriert sein, damit bei Bedarf hierher gesprungen wird!#
    #==============================================================================================================#
    
doprint=False
printresult=''
db_name='data/absent.sq3'

if __name__=="__main__":
    db_create()
    doprint=True
    if len(sys.argv)==2:
        if sys.argv[1]=="resetadmin":
            lege_admin_an()
            prt('administrativer Login mit: admin und Passwort pass8 eingerichtet!')

    try: # nur unter Linux erfolgversprechend
        subprocess.call('chown :www-data '+db_name,shell=True)
        subprocess.call('chmod 0660 '+db_name,shell=True)
    except:
        pass
