#!/usr/bin/python3
# -*- coding: utf-8 -*-
'''
________________________________________________________________________
 a_tabelle.py                                                         
 This file is free software under         
     the terms of the GPL without any warranty - see the file COPYING 
 Diese Datei ist  Freie Software gemäß
             der GPL ohne jegliche Garantie - siehe die Datei COPYING 
________________________________________________________________________

 a_tabelle.py 
________________________________________________________________________
'''

#import codecs

from a_konstanten import *

import a_tool
import a_db
import a_session as a_s

import a_termine

def schueler_kalender(pid):
    row=a_db.get_one_rowdict('SELECT  name,apdatum FROM personen WHERE pid=?',(pid,))
    if row:
        name,apdatum=row
    else:
        return "Schüler existiert nicht"
    abdict={}
    rowdict_list=a_db.get_rowdict_list('''
                SELECT  datum,status,attestpflicht,kommentar,teilflag
                FROM    absenzen
                WHERE   pid=? AND datum>=? AND datum<=?
                ORDER BY datum''',(pid,a_s.schuljahresgrenze_iso(),a_s.schuljahresgrenze_iso(1)))
    for row in rowdict_list:
        abdict[ row["datum"] ]=[row["status"],row["attestpflicht"],row["kommentar"],row["teilflag"]]
                
    liste=[]
    liste.append('<h3>Übersicht über den Schulbesuch von '+name+'</h3><div class=kalender>')
    liste.append('<table class="alternierend" style="margin:0;padding:0;border-style:solid;border-width:1px">')
    
    line=['<tr><th>Tage</th>']
    for i in range(1,32):
        line.append('<th style="width:1.2em;">'+str(i)+'</th>')
    line.append('<th><small>Abwesend</small></th>')
    line.append('</tr>')
    liste.append("".join(line))
    startjahr=int(a_s.schuljahresgrenze_iso()[:4])
    at_timestamp=a_tool.timestamp
    heute=a_tool.heutedatum
    j_tage=0
    j_abs=0
    for jahrdelta,monat in [(0,9),(0,10),(0,11),(0,12),(1,1),(1,2),(1,3),(1,4),(1,5),(1,6),(1,7)]:
        jahr=startjahr+jahrdelta
        m_tage=0
        m_abs=0
        line=['<tr><td align=right>'+monatsname[monat][:3]+'&nbsp;'+str(jahr)+'</td>']
        schalttag=1 if (monat==2 and jahr%4==0) else 0
        for i in range(1,32):
            if i>monatstage[monat]+schalttag:
                line.append('<td> </td>')
            else:
                datum='{:4}-{:02}-{:02}'.format(jahr,monat,i)
                if datum>heute:
                    line.append('<td> </td>')
                else:
                    ob,warum=a_tool.ist_freier_tag_warum(datum)
                    if ob:
                        line.append('<td class="frei"><tooltip>&nbsp;<span class="ferien">'+warum+'</span></tooltip></td>')
                    elif ist_wochenende(datum):
                        line.append('<td class="saso"> </td>')
                    else:
                        m_tage+=1
                        if datum in abdict:
                            m_abs+=1
                            if abdict[datum][0]==LEVEL_SCHULBEFREIUNG:
                                klasse='leer'
                            elif (apdatum<=datum or abdict[datum][1]) and abdict[datum][0]<LEVEL_ATTESTOK:
                                klasse='afehlt'
                            elif abdict[datum][0]<LEVEL_ENTSCHULDIGUNG:
                                klasse='fehlt'
                            else: klasse='absenz'
                            kommentar=abdict[datum][2]
                            line.append('<td class="'+klasse+'"><tooltip>'+STATUSCHAR[abdict[datum][0]]+'<span>'+kommentar+'</span></tooltip></td>')
                        else:
                            line.append('<td class="leer">.</td>')
        if m_tage>0:
            line.append('<td>'+str(m_abs)+'/'+str(m_tage)+'='+str(int(100.0*m_abs/m_tage))+'%</td></tr>')
        else:
            line.append('<td> </td></tr>')
        liste.append("".join(line))
        j_tage+=m_tage
        j_abs+=m_abs
    
    liste.append('</table>\n</div>\n')
    if j_tage>0:
        liste.append('<p>Gesamtfehlquote:'+str(j_abs)+'/'+str(j_tage)+'='+str(int(100.0*j_abs/j_tage))+'%</p>')
    liste.append('<p>Bei der Quote wird hier jede Art von Absenz (auch Schulbefreiung und halbtägige Absenzen) gezählt</p>')
    datei="\n".join(liste)
    return datei


def export_absenzentabelle(leere_liste=False):
    kbez=a_s.aS.kurs
    kopf,seite,fuss,seitenseparator=a_tool.lese_htmlvorlage_ein('kursliste.html')
    liste=absenzentabelle(0,kbez,True,leere_liste)
    datei=kopf+seite.replace('ABKURSLISTE',liste).replace('ABKBEZ',kbez)+fuss
    return datei


def absenzentabelle(monat=0,kriterium="A",exporthtml=False,leere_liste=False):
    global alphabetisch
    a_tool.starttiming("absenzentabelle")
    ausgabe=['<div class="b1">'] if not exporthtml else []
    footer=''
    mit_quote=False
    at_timestamp=a_tool.timestamp
    at_now=a_tool.now
    ineinerwoche_datum=a_tool.timestamp2isodatum(a_tool.ts_verschiebe(at_timestamp,7))
    
    ist_alphabetisch=False # Die alphabetischen Anzeigen sind Verwaltern vorbehalten - diese werden dann nach Stufen gefiltert!

    # monat, jahr geben den hervorzuhebenden Monat ( i.d.R. der Mittlere von Dreien) an
    # aus Monat wird das Jahr berechnet
    if monat==0: 
        monat= at_timestamp.tm_mon #monat:1-12
    if monat<=8: # wenn jetzt 1. HJ, Anzeige soll 2. HJ zeigen: also früher im Schuljahr = letztes Kalenderjahr
        jahr=a_s.aS.anzeigeschuljahr+1
    else:
        jahr=a_s.aS.anzeigeschuljahr
    minjsdatum=['2000-01-01','2000-01-01','2000-01-01','2000-01-01','2000-01-01','2000-01-01','2000-01-01','2000-01-01','2000-01-01','2000-01-01','2000-01-01','2000-01-01','2000-01-01','2000-01-01']
    for i in range(14): 
        #if not a_s.aS.ist_seki_in(i):
        mindatum=''
        if i<11:    mindatum=a_db.get_config('intern_mindestdatum-gum')
        if i==11:   mindatum=a_db.get_config('intern_mindestdatum-q11')
        if i==12:   mindatum=a_db.get_config('intern_mindestdatum-q12')
        if a_tool.istisodatum(mindatum):
            minjsdatum[i]=mindatum
    
    pliste=[]
    starttag=0  # "unkonfiguriert" 
    anzahl=0    # "unkonfiguriert"
    ## Welche Namen sollen gezeigt werden? 
    if len(kriterium)>1 and kriterium[0]=="@": # oh, ein Klassenname
        klassenname=kriterium[1:]
        schueler_rowdict_list=a_db.get_rowdict_list('''
            SELECT  personen.pid,personen.name,klasse,js,apdatum 
            FROM    personen
            WHERE   personen.klasse=?
            ORDER BY personen.sortiername''',(klassenname,))
    
    # solche Namen, die Absenzen in den letzen 14 Tagen erhalten haben? (genauer: FRIST_FUER_AKTUELLE)
    elif kriterium==alphabetisch[0]: # Mörderhack für den ersten Eintrag ("Aktuelle")
        ist_alphabetisch=True
        ts=a_tool.ts_verschiebe(at_timestamp,-FRIST_FUER_AKTUELLE)
        startdatum=a_tool.timestamp2isodatum(ts)
        startjahr=ts.tm_year
        startmonat=ts.tm_mon
        starttag=ts.tm_mday
        schueler_rowdict_list=a_db.get_rowdict_list('''
            SELECT DISTINCT
                    personen.pid,name,klasse,js,apdatum 
            FROM    personen, absenzen
            WHERE   personen.pid=absenzen.pid AND datum>=?
             AND js BETWEEN ? AND ?
             AND (    (status BETWEEN ? AND ? AND  attestpflicht=0 AND (apdatum>datum or apdatum<"2012-00-00"  )      )
                   OR   (status BETWEEN ? AND ? AND (attestpflicht=1 OR  (apdatum<=datum AND apdatum>"2012-00-00") )    ) )
            ORDER BY sortiername''',(startdatum, \
            a_s.aS.von_js,a_s.aS.bis_js, \
            LEVEL_VERMISST, LEVEL_KRANKMELDUNG, \
            LEVEL_VERMISST, LEVEL_ENTSCHULDIGUNG \
            ))
    # alle Namen mit bestimmten Anfangsbuchstaben?
    elif kriterium in alphabetisch or kriterium=='Alle':
        ist_alphabetisch=True
        if kriterium=='Alle':
            von='A'
            bis='Zzz'
        else:   
            von=kriterium[0]
            i=alphabetisch.index(kriterium)
            bis=(alphabetisch+['Ä'])[i+1][0] # Ä kommt nach Z, in den sortiernamen kommen aber keine Umlaute mehr vor
        schueler_rowdict_list=a_db.get_rowdict_list('''
            SELECT  personen.pid,name,klasse,js,apdatum
            FROM    personen 
            WHERE   sortiername BETWEEN ? AND ?
                    AND      js BETWEEN ? AND ?
            ORDER BY sortiername''',(von.lower(),bis.lower(),a_s.aS.von_js,a_s.aS.bis_js))
    # alle in einem bestimmten Kurs?
    elif len(kriterium)>1: # ok, dann ist es einfach ein Kursname
        if not exporthtml: footer+='''<br>
            <div style="float:left;">
                <form action="a_main.py" method="post" target="_blank">
                    Eine leere Kursliste in eigenem Fenster <input value="ausgeben" type="submit">
                    <input name="popupaktion" value="kursliste_leer" type="hidden">
                </form>
            </div><div style="float:left;width:3em;text-align:center;"> </div>
            <div style="float:left;">
                <form action="a_main.py" method="post" target="_blank">
                    Komplette Kurs-Absenzenliste in eigenem Fenster <input value="ausgeben" type="submit">
                    <input name="popupaktion" value="kursliste_absenzen" type="hidden">
                </form>
            </div><div style="clear:left;"></div>'''
        if a_s.aS.get_tagesfilter() or exporthtml:
            kurstage=a_db.get_first_element('SELECT wochentage FROM kurse WHERE kbez=?',(kriterium,),default=31) # aus der DB die Tage holen, 
            if a_s.aS.get_tagesfilter()==2 or exporthtml: # jetzt werden alle Tage seit Schuljahres- bzw. Kalenderjahrbeginn angezeigt
                mit_quote=True
                startjahr,startmonat,starttag=a_tool.letzter_halbjahresbeginn()
                anzahl=400 # limitiert bis nächste Woche
        schueler_rowdict_list=a_db.get_rowdict_list('''
            SELECT  personen.pid,personen.name,klasse,js,apdatum 
            FROM    personen,kurse,p2k 
            WHERE   personen.pid=p2k.pid 
                AND kurse.kbez=p2k.kbez 
                AND kurse.kbez=? 
            ORDER BY personen.sortiername''',(kriterium,))
    # oder garnichts?
    else:
        return ""
        
    # Alle gefundenen Schüler in eine Liste eintragen
    ist_irgendwo_verwalter=False
    letzte_js=0
    letzte_klasse=''
    for row in schueler_rowdict_list: 
        pid,name,klasse,js,apdatum = row
        pliste.append([pid,name,klasse,js,apdatum])
        if js>0 and not ist_irgendwo_verwalter and js!=letzte_js: # kleine Optimierung...
            letzte_js=js
            ist_irgendwo_verwalter=a_s.aS.ist_verwalter_in(js)
    # Tabelle beginnen - ggf. wurde bei echtem Kurs vorher schon Button eingeblendet
    ausgabe.append('<table class="alternierend"><tr><th colspan="2">Name</th>')
    if (ist_irgendwo_verwalter or a_s.aS.genug_rechte(RECHTE_ABSENZENBLATT)) and  not exporthtml: # Ein Verwalter bekommt zusätzliche statistische Informationen, auch wenn er dort kein Verwalter ist...
        ausgabe.append('<th><small>Gesamt<br></small></th><th><small>Quote</small></th><th><small>Entsch.<br>fehlt</small></th><th><small>Attest<br>fehlt</small></th>')
    # Alle anzuzeigenden Kalentertage auflisten (in Liste und in oberster Tabellenzeile)
    datumsliste=[]
    wotagname=  ['Mo','Di','Mi','Do','Fr','Sa','So']
    auszublenden=[0,   0,   0,   0,   0,   1,   1]
    if a_s.aS.get_tagesfilter() and kurstage!=31:  #nur wenn Tage fehlen
        for i in range(5):              #alle Tage abklappern
            if not (kurstage& 1<<i):    #ist das Bit des Tages nicht gesetzt?
                auszublenden[i]=1       #dann ausblenden lassen
    wurde_ausgeblendet=False
    
    if starttag==0: # noch unkonfiguriert, z.B. durch "Aktuelle"
        startmonat=monat-(1 if (not a_s.aS.ist_verwalter_in(0) or monat==9) else 2) # darf wohl auch negativ sein
        startjahr=jahr
        anzahl=69 if not a_s.aS.ist_verwalter_in(0) else 93
        if startmonat==9:
            starttag=1
        elif startmonat==8:
            startmonat=9
            starttag=1
        else:
            starttag=10 if at_timestamp.tm_mday<15 else 17; 
    else:
        #start monat muss mit starttag gesetzt worden sein
        if not anzahl:
            anzahl=FRIST_FUER_AKTUELLE+15 
    zum_wochenanfang=a_tool.ts_erstellen(startjahr,startmonat,starttag).tm_wday
    starttag-=zum_wochenanfang
    anzahl+=zum_wochenanfang
    if not leere_liste:
        #######################################################################################
        # Schleife über alle anzuzeigenden Tage mit Festlegung von Aussehen und Sichtbarkeit  #
        #######################################################################################
        wochenanfang=False
        wurde_geskipt=False
        for i in range(anzahl):
            ts=a_tool.ts_erstellen(startjahr,startmonat,starttag+i)
            if ts.tm_wday==0:
                wochenanfang=True
            if not auszublenden[ts.tm_wday]:
                datum=a_tool.timestamp2isodatum(ts)
                #a_tool.prt(a_s.aS.get_lesbares_schuljahr()+' '+datum+'. ')
                if anzahl>=400 and datum>=ineinerwoche_datum:
                    break
                
                skip_wegen_ferien=a_tool.ist_freier_tag(datum)
                
                
                if not skip_wegen_ferien:
                    # cssclass enthält am Ende dieses Blocks alle css-classen, die auch die folgenden Zeilen berücksichtigen sollen
                    #  und wird in die DatumsDatenstruktur eingebunden
                    # was nur im Spaltenkopf gelten soll wird mit vereinige_klassen dynamisch konkateniert
                    if wurde_geskipt:
                        cssclass='atwg' # wurde geskipt
                        wurde_geskipt=False
                    elif wochenanfang: # Also auch an einem Dienstag, wenn er der erste anzuzeigende Tag ist!
                        cssclass='atmo' # absenzentabellemontag
                    else:
                        cssclass='atwa' if wurde_ausgeblendet else ''
                    if exporthtml:
                        cssclass+=' atkl '
                    if (not exporthtml) and ts.tm_mday==at_timestamp.tm_mday and ts.tm_mon==at_timestamp.tm_mon and ts.tm_year==at_timestamp.tm_year:
                        cssclass+=' athe'
                        ausgabe.append('<th '+vereinige_klassen('atsize','atti',cssclass))
                        bg='' #'lightblue'
                    elif ts.tm_wday>=5:
                        cssclass+=' variabel atwe'
                        ausgabe.append('<th '+vereinige_klassen('atsize',cssclass))
                        bg='' #'#222'
                    elif ts.tm_mon!=monat and not exporthtml:
                        cssclass+=' variabel atam' if a_s.aS.get_tagesfilter()!=2 else ' atam'
                        ausgabe.append('<th '+vereinige_klassen('atsize',cssclass,'atamti'))
                        bg='-'
                    else:
                        ausgabe.append('<th '+vereinige_klassen('atsize','atti',cssclass))
                        bg=''
                    ausgabe.append('>')
            
                    if a_s.aS.ist_seki_in(0):
                        titel='"Öffne eine Liste aller Absenzen vom '+a_tool.isodatum2datum(datum)+'\n(Schultag Nummer '+str(a_s.schultage_bis_heute(datum))+') in neuem Fenster"'
                        ausgabe.append('<a href="a_main.py?popupaktion=tagesliste&tageslistendatum='+datum+'" target="_blank" title='+titel+'>')
                    
                    ausgabe.append(str(ts.tm_mday)+'.<br>'+ str(ts.tm_mon)+'.<br>'+wotagname[ts.tm_wday]+'</th>')
                    if a_s.aS.ist_seki_in(0):
                        ausgabe.append('</a>')
                    datumsliste.append([datum,wotagname[ts.tm_wday],bg,cssclass])
                    wurde_ausgeblendet=False
                    wochenanfang=False
                    
                else:
                    wurde_geskipt=True
            else:
                wurde_ausgeblendet=True
            
        if mit_quote:
            ausgabe.append('<th '+vereinige_klassen('atkl atti')+'><small>Quote<br>bis<br>heute</small></th>')
    else:
        ausgabe.append('<th> </th>'*6)
    ausgabe.append('</tr>\n')
    ersterbuchstabe=""
    zeilennummer=0
    naechstmoeglichezeile=minimaleranfangsbuchstabenmarkierungsabstand
    panzahl=len(pliste)
    #TTTTTTTTTTTTTTTTTTTTTTTTTTTTTTTTTTTT
    # Terminzeile
    #TTTTTTTTTTTTTTTTTTTTTTTTTTTTTTTTTTTT
    terminbetroffene_dict={} # wird 'datum':{BetroffenenMenge} enthalten
    if not leere_liste and a_db.get_config_int('flag_termineanzeigen')>0:
        extraclass='atterminzeile ' # Highlighten des Zeile, kann wg. Cascading überschrieben werden, was ok ist
        ausgabe.append('\t<tr><td colspan="2" '+vereinige_klassen(extraclass)+' style="text-align:center;"><b>Termine</b></td>')
        if (ist_irgendwo_verwalter or a_s.aS.genug_rechte(RECHTE_ABSENZENBLATT)) and not exporthtml:
            ausgabe.append('<td colspan=4 '+vereinige_klassen(extraclass)+'> </td>')
        #TTTTTTTTTTTTTTTTTTTTTTTTTTTTTTTTTTTT
        # Innere Schleife über die Zellen in der Terminzeile
        #TTTTTTTTTTTTTTTTTTTTTTTTTTTTTTTTTTTT
        for [datum,wochentag,background,cssclass] in datumsliste:
            termin_obj_liste=a_termine.A_Termin.lade_tages_liste((0,1,3,4),datum) # Liste mit Terminobjekten für diesen Tag
            terminliste=''
            terminzaehler=0
            terminanzeige='&nbsp;'
            ist_sa_tag=False
            gibt_geloeschte=False
            betroffene=''
            #tttttttttttttttttttttttttttttttttttt
            #tt Schleife über alle Termine an dem Tag
            #tttttttttttttttttttttttttttttttttttt
            for loop_termin in termin_obj_liste:
                if kriterium[0]=="@":    #wenn UM-Klasse:
                    if not (  kriterium in loop_termin.betroffene.split() or loop_termin.typ==9): 
                        continue    # Wenn Klasse nicht vorkommt wird sie ignoriert!
                elif a_s.aS.get_aktjs()>10:
                    abbrechen=True
                    for bkurs in loop_termin.betroffene.split():
                        if not bkurs.startswith("@"):
                            abbrechen=False
                    if abbrechen:
                        continue
                geloescht=True if loop_termin.status<0 else False
                if loop_termin.typ in [1,3,4] and not geloescht:
                    ist_sa_tag=True 
                    betroffene+=' '+str(loop_termin.betroffene)
                if geloescht and not gibt_geloeschte:
                    terminliste+='<small>'
                    gibt_geloeschte=True
                terminliste+=loop_termin.als_popup()
                terminzeichen=loop_termin.name[:4] # nur die ersten 4 Zeichen werden in der Tabelle angezeigt
                if geloescht:
                    if terminzaehler==0: # sonst würde es sowieso ersetzt werden
                        terminanzeige='<geloescht>('+terminzeichen+')</geloescht>'
                    else:
                        pass # das vorherige bleibt bestehen: ein aktueller nichtgelöschter Termin bleibt lesbar
                else:
                    terminanzeige='<schwarz>'+terminzeichen+'</schwarz>'
                if not geloescht:
                    terminzaehler+=1
            #ttttttt ende der Schleife über die Termine eines Tages, was bleibt also für diesen Tag an Terminen?
            if gibt_geloeschte:
                terminliste+='</small>'
            if terminzaehler>1:
                terminanzeige='!' if terminzaehler>9 else str(terminzaehler)
                cssclass=' attermin '+cssclass
            if ist_sa_tag:
                cssclass+=' atterminSA' 
                terminbetroffene_dict[datum]=set(betroffene.split()) # lege eine Menge (python set) mit Kursen an, die von der/den Schulaufgabe(n) betroffen sind
            if gibt_geloeschte:
                cssclass+=' attermingg'
            ausgabe.append( ('<td '+vereinige_klassen('atsize',extraclass,cssclass,'center')+' style="background:'+background+';'+'">') if len(background)>1 else \
                            ('<td '+vereinige_klassen('atsize',extraclass,cssclass,'center')+                                      '>' ) )
            ## Darf man überhaupt Termin eintragen?
            extralinkinfo=''
            if datum>a_tool.heutedatum: 
                    ausgabe.append('<a class="termintip " href="a_termine.py?verwalten=nein&editdatum='+datum+'">')
                    extralinkinfo='Neuen Termin für '+a_tool.isodatum2datum(datum)+' eintragen'
            else:
                ausgabe.append('<a class="termintip nopointer" >')

            ausgabe.append('<minilink>'+terminanzeige+'</minilink>')
            if terminzaehler or gibt_geloeschte:
                ## CSS-PopUp-Inhalt
                ausgabe.append('<span>'+wochentag+', '+a_tool.isodatum2datum(datum)+'<br>'+terminliste+'</span>')
            elif extralinkinfo:
                ausgabe.append('<span>'+extralinkinfo+'</span>')
            ausgabe.append(' </a></td>')
        #Ende der Zusammenstellung der Termin-Zeile
        if mit_quote:
            ausgabe.append('<td '+vereinige_klassen(extraclass)+'> </td>')
        ausgabe.append('</tr>\n')
        extraclass=''
    ######################################
    # Die Schleife über die Zeilen - also über alle Schüler
    ######################################
    for [pid,name,klasse,js,apdatum] in pliste: 
        s_zeile=[]
        if ist_alphabetisch and a_s.aS.ist_berechtigt_in[js]<4:
            continue # überspringe diese Zeile für nur mit selektiven Verwalterrechten ausgestattete User
        zeilennummer+=1
        absenzenzahl=0
        if ersterbuchstabe and name[0]!=ersterbuchstabe and zeilennummer>=naechstmoeglichezeile:
            extraclass='attl' # A bsenzen T abelle T rennungs L inie
            naechstmoeglichezeile=zeilennummer+minimaleranfangsbuchstabenmarkierungsabstand
        else:
            extraclass=''
        if exporthtml:
            extraclass+=' atkl '
        ersterbuchstabe=name[0]
        if pid==a_s.aS.highlight:
            extraclass='athl '+extraclass # Highlighten des Zeile, kann wg. Cascading überschrieben werden, was ok ist
            tr='<tr id="Highlight">'
        else:
            tr='<tr>'
        s_zeile.append('\t'+tr+'<td '+vereinige_klassen(extraclass)+'>'+str(zeilennummer)+'</td><td '+vereinige_klassen(extraclass)+'>')
        if (ist_irgendwo_verwalter or a_s.aS.genug_rechte(RECHTE_ABSENZENBLATT)) and not exporthtml:
            s_zeile.append('<a href="a_main.py?popupaktion=absenzenbrief;einzelschueler='+str(pid)+'" target="_blank">')
            s_zeile.append(name+'</a><span style="float:right">&nbsp;')
            s_zeile.append('('+klasse+')</span></td>')
            strg,dummy,dummy,dummy,dummy,dummy=get_einzelstatustabelle(pid,apdatum,'',extraclass) 
            s_zeile.append(strg)
        else:
            s_zeile.append(name+'</td>')
        
        if leere_liste:
            s_zeile.append(('<td width=14%'+vereinige_klassen(extraclass)+'>&nbsp;</td>')*6)
        else:
            # alle anzuzeigenden Absenzen des Schülers lesen
            abdict={}
            g_dict={}
            rowdict_list=a_db.get_rowdict_list('''
                SELECT  datum,status,attestpflicht,kommentar,teilflag
                FROM    absenzen
                WHERE   pid=? AND datum>=? AND datum<=?
                ORDER BY datum''',(pid,datumsliste[0][0],datumsliste[-1][0]))
            for row in rowdict_list:
                tag,status,apflicht,kommentar,teilflag =row
                abdict[tag]=[status,apflicht,kommentar,teilflag]
            if a_s.aS.ist_verwalter_in(0) or a_s.aS.ist_seki_in(0) or a_s.aS.ist_klassleiter():
                rowdict_list=a_db.get_rowdict_list('''SELECT datum,klarname,kommentar FROM gespraeche WHERE pid=?''',(pid,))
                for row in rowdict_list:
                    datum,klarname,kommentar=row
                    if kommentar:
                        kommentar=' ('+kommentar+')'
                    g_dict[datum]='Gespräch geführt von '+klarname+kommentar
            # alle Kurse, in denen der Schüler ist, einlesen und in eine Menge schreiben
            alleseinekurse_list=['@'+klasse] # UM-Schüler? Dann ist seine Klasse der Kurs...
            item_list=a_db.get_column_list('''SELECT kbez FROM p2k WHERE pid=?''',(pid,))
            for kurs in item_list:
                alleseinekurse_list.append(kurs)
            alleseinekurse_set=set(alleseinekurse_list)
            alleseinekurse_list=[]
            ###############################################
            # Innere Schleife über die Zellen in der Zeile
            ###############################################
            for [datum,wochentag,background,cssclass] in datumsliste:
                # SA-termin? betroffen?
                von_sa_betroffen=False
                if datum in terminbetroffene_dict:
                    betroffenin=alleseinekurse_set & terminbetroffene_dict[datum] # Schnittmenge, das das dict Mengen enthält
                    if len(betroffenin)>0: von_sa_betroffen=True
                    
                attestpflicht=0
                head=name
                #variabel= '' if (len(background)==0 or background=='lightblue' ) else 'variabel'
                if len(apdatum)==10 and datum>=apdatum:
                    if wochentag[0]!="S": 
                        background='#CBA'
                    attestpflicht=1
                    head+=' <small>(Attestpfl. ab '+a_tool.isodatum2datum(apdatum)+')</small>'
                status=0
                zeitraum=''
                if datum in abdict:
                    status=abdict[datum][0]
                    background=STATUSBG[status] 
                    if background=='green' and abdict[datum][3]:
                        background='lightgreen'
                    if abdict[datum][1]!=0:
                        attestpflicht=1
                    if status>=LEVEL_ATTESTOK:
                        attestpflicht=2
                    if attestpflicht==1 and status>LEVEL_SCHULBEFREIUNG:
                        background='red'
                    if status>=LEVEL_VERMISST:
                        absenzenzahl+=1
                    if status>=LEVEL_ENTSCHULDIGUNG:
                        zeitraum=' (stundenweise)' if abdict[datum][3] else ' (ganztags)'
                g_meldung=''
                if datum in g_dict:
                    g_meldung='<br><b>'+g_dict[datum]+'</b>'
                    background='#CC3'
                #s_zeile.append( \
                #        ('<td '+vereinige_klassen(extraclass,cssclass,'center')+' style="background:'+background+';'+'">') if len(background)>1  else \
                #        ('<td '+vereinige_klassen(extraclass,cssclass,'center')+'>' ) )
                s_zeile.append( \
                        ('<td {} style="background: {};">'.format(vereinige_klassen('atsize',extraclass,cssclass,'center'),background) ) if len(background)>1  else \
                        ('<td {} >'.format(vereinige_klassen('atsize',extraclass,cssclass,'center')) ) )
                # Darf man überhaupt Aktion ausführen?
                if (  status<a_s.aS.aktion or status==LEVEL_VERMISST ) and datum>=minjsdatum[js] and \
                    not (a_s.aS.aktion==LEVEL_VERMISST and (datum>a_tool.heutedatum)): # man kann nicht in der Zukunft vermissen!
                        s_zeile.append('<a class="tooltip" href="a_main.py?pid={};datum={}"> '.format(str(pid),datum))
                else:
                    s_zeile.append('<a class="tooltip nopointer" > ')
                if von_sa_betroffen: s_zeile.append('<unterstreichen>')
                if datum in abdict:
                    # Anzuzeigender Kennbuchstabe
                    s_zeile.append(STATUSCHAR[status])
                    if g_meldung:
                        s_zeile.append('g')
                    if von_sa_betroffen: s_zeile.append('</unterstreichen>')
                    ## CSS-PopUp-Inhalt
                    anzeigekommentar=komprimiere_kommentar(abdict[datum][2])
                    topstrg=''  if (zeilennummer<6 or panzahl-zeilennummer>5) \
                                else 'style="top:-'+str(int(6-(panzahl-zeilennummer)+len(anzeigekommentar.split('<br>'))))+'em;"'
                    #s_zeile.append('<span '+topstrg+' ><b>'+head+'</b>'+wochentag+', '+a_tool.isodatum2datum(datum)+g_meldung+zeitraum+'<br>'+STATUSSTRGLONG[status]+'<br>')
                    s_zeile.append('<span {}><b>{}</b>{}, {}{}{}<br>{}<br>'.format(topstrg,head,wochentag,a_tool.isodatum2datum(datum),g_meldung,zeitraum,STATUSSTRGLONG[status]))
                    # Attestpflicht dokumentieren
                    if attestpflicht==1  and status>LEVEL_SCHULBEFREIUNG:
                        s_zeile.append('<b style="color:red">{}</b>'.format(atteststrg))
                    s_zeile.append('<br><i>{}</i>'.format(anzeigekommentar))
                    if von_sa_betroffen: s_zeile.append('<br><rot>Schulaugabe in {}</rot>'.format (" ".join(betroffenin)))
                    s_zeile.append('</span>')
                else:
                    if g_meldung:
                        s_zeile.append('g')
                    else:
                        s_zeile.append("&nbsp;")
                        g_meldung='<br>(keine Absenz)'
                    if von_sa_betroffen:
                        s_zeile.append('</unterstreichen>')
                    topstrg='' if (zeilennummer<6 or panzahl-zeilennummer>3) else 'style="top:-'+str(int(3-panzahl+zeilennummer))+'em;"'
                    #s_zeile.append('<span '+topstrg+' ><b>'+head+'</b>'+wochentag+', '+a_tool.isodatum2datum(datum)+g_meldung)
                    s_zeile.append('<span {}><b>{}</b>{}, {}'.format(topstrg,head,wochentag,a_tool.isodatum2datum(datum)+g_meldung))
                    if von_sa_betroffen: s_zeile.append('<br><rot>Schulaugabe in {}</rot>'.format (" ".join(betroffenin)))
                    s_zeile.append('</span>')
                s_zeile.append(' </a></td>')
        #Ende der Zusammenstellung einer Zeile
            if mit_quote:
                s_zeile.append('<td '+vereinige_klassen(extraclass)+'>'+str(int(round(absenzenzahl*100.0/len(datumsliste))))+'%</td>')
        ausgabe.append(''.join(s_zeile)+'</tr>\n')
    ausgabe.append('</table>\n')
    ausgabe.append(footer)
    if not exporthtml:
        ausgabe.append('</div>')
    a_tool.endtiming('absenzentabelle')
    return ''.join(ausgabe)
    
def vereinige_klassen(*args):         
    alleklassen=' '.join(list(args)).strip()
    if alleklassen:
        return ' class="{}" '.format(alleklassen)
    return ''
    
def ist_wochenende(datum):
    if a_tool.wochentag(datum)>=5:
        return True
    return False
        
def komprimiere_kommentar(text):
    zeilen=text.split("<br>")
    ergebnis=[]
    for zeile in zeilen:
        if len(ergebnis)>0:
            ergebnis.append('<br>')
        if len(zeile)>35:
            woerter=zeile.split(" ",2)
            if len(woerter)>2:
                ergebnis.append('{} {} <small>{}</small>'.format(woerter[0],woerter[1],woerter[2]))
            elif len(woerter)>1:
                ergebnis.append('{} <small>{}</small>'.format(woerter[0],woerter[1]))
            else:
                ergebnis.append('<small>{}</small>'.format(zeile))
        else:
            ergebnis.append(zeile)
    return "".join(ergebnis)

def get_einzelstatustabelle(pid,apdatum,background='',extraclass=''):
    '''gibt einen string mit drei Tabellenzellen zurück
     (mit den Anzahlen der gesamten Absenzen, der ohne Krankmeldung und der fehlenden Atteste)
    dann folgen die ersten drei zahlen als integer.
    Zu nutzen: strg,absenzen,quote,unentschuldigt,attestfehlt,ganztaegige=get_einzelstatustabelle(pid,apdatum)'''
    von,bis=a_s.schuljahresgrenze_iso(),a_s.schuljahresgrenze_iso(True)
    if background=='':
        td_def='<td class="center '+extraclass+'">'
    else:
        td_def='<td class="center '+extraclass+'" style="background:'+background+';">'
    
    if len(apdatum)!=10:
        apdatum="9999-99-99"
    ergebnis=''
    absenzen=a_db.get_first_element('''
        SELECT  count(datum) 
        FROM    absenzen 
        WHERE   absenzen.pid=? AND status>=?
         AND datum BETWEEN ? AND ?''',(pid,LEVEL_VERMISST,von,bis), default=0)
    
    ergebnis+=td_def+str(absenzen)+'</td>'
    quote=int(round(absenzen*100/max(a_s.schultage_bis_heute(),1)))
    ergebnis+=td_def+str(quote)+'<small>&#x202F;%</small></td>'
    
    unentschuldigt=a_db.get_first_element('''
        SELECT  count(datum) 
        FROM    absenzen 
        WHERE   absenzen.pid=? AND status>=? AND status<?
         AND attestpflicht=0 AND (?>datum)
         AND datum BETWEEN ? AND ?''',(pid,LEVEL_VERMISST,LEVEL_ENTSCHULDIGUNG,apdatum,von,bis), default=0)
    
    if unentschuldigt:
        ergebnis+='<td class="red '+extraclass+'">'+str(unentschuldigt)+'</td>'
    else:
        ergebnis+=td_def+'0</td>'
    attestfehlt=a_db.get_first_element('''
        SELECT  count(datum) 
        FROM    absenzen 
        WHERE   absenzen.pid=? AND status>=? AND status<? 
            AND ( attestpflicht=1 OR ?<=absenzen.datum )
         AND datum BETWEEN ? AND ?
        ''',(pid,LEVEL_VERMISST,LEVEL_ATTESTOK,apdatum,von,bis), default=0)
    if attestfehlt:
        ergebnis+='<td class="red '+extraclass+'">'+str(attestfehlt)+'</td>'
    else:
        ergebnis+=td_def+'0</td>'
        
    ganztaegige=a_db.get_first_element('''
        SELECT  count(datum) 
        FROM    absenzen 
        WHERE   absenzen.pid=? AND teilflag==0 AND status>=?
         AND datum BETWEEN ? AND ?''',(pid,LEVEL_VERMISST,von,bis), default=0)
    
    return ergebnis,absenzen,quote,unentschuldigt,attestfehlt,ganztaegige
            


